==> javathrottle-0.4.tar.gz: javathrottle-0.4/src/net/f00f/javathrottle/Main.java <==
package net.f00f.javathrottle;
/*
	JavaThrottle, a program to limit the bandwidth between an input
	stream and an output stream.  Copyright (C) 2003 Ben Damm

	For information, contact bdamm@f00f.net

                        -----  LGPL License  -----
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                        -----       END      -----

*/


import java.awt.Component;
import java.awt.Dimension;
//import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.ContainerAdapter;
import java.awt.event.ContainerEvent;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.Map;
import java.util.Observer;
import java.util.Observable;
import java.util.Properties;

import javax.swing.*;

import net.f00f.javathrottle.layout.TableLayout;

/**
 * This class does the command line switches and the GUI.  The
 * throttling of the pipe is done in a Throttle implementation.
 *
 * @author Ben Damm
 * */
// Rates are bytes per millisecond, throughout the code.
public class Main extends JPanel implements ActionListener
{
	private static String KEY_desiredRate = "desiredRate";
	private static String KEY_persist = "persist";

	private JLabel bytesLabel;
	private JLabel rateLabel;
	private JLabel timeLabel;
	private JLabel bufLabel;
	private JLabel bufFillLabel;
	private JProgressBar windowLabel;
	private JLabel desiredWindowLabel;
	private JLabel windowMaxReadLabel;
	private JCheckBox stayOpenBox;
	private JWindowSelect desiredWindowSelect;
	private JRateSelect desiredRateSelect;
	private JButton pauseBtn;
	private JButton stopBtn;
	private boolean stayOpenDefault = false;

	private List activeComponents;
	// The master component
	private Component mainComponent;

	private Throttle throttle;

	private static String VER_project = "JavaThrottle";
	private static String VER_version = Version.VERSION + " " + Version.BUILD_DATE;
	private static String VER_author = "Ben Damm, bdamm@f00f.net";
	private static String VER_url = "http://dammfine.com/projects/javathrottle/";

	private static boolean DEBUG_enable = false;

	/**
	 * Entry point.
	 * */
	public static void main( String args[] )
	{
		// Parse cmd line
		final Properties p = parseCmdLine( args );

		// Error
		if( p == null )
		{
			usage();
			return;
		}

		// The window skeleton
		final JFrame jf = new JFrame("JavaThrottle");

		// When the user closes the window, kill the application.
		jf.addWindowListener(new WindowAdapter()
				{
					public void windowClosing( WindowEvent we )
					{
						System.exit(0);
					}
				});

		Runnable runner = new Runnable()
		{
			public void run()
			{
				// Create the main class
				Main mainPane = new Main(p);

				mainPane.addContainerListener(new ContainerAdapter()
						{
							public void componentAdded( ContainerEvent ce )
							{
								// Resize to fit the components
								jf.pack();
							}
						});

				// Add it
				jf.getContentPane().add( mainPane );
				jf.pack();
				jf.show();

				// Start the magic
				mainPane.run();
			}
		};

		try
		{
			SwingUtilities.invokeAndWait( runner );
		}
		catch( java.lang.reflect.InvocationTargetException ite )
		{
			// ??
		}
		catch( InterruptedException ie )
		{
			return;
		}
	}


	public static JButton makeSetButton()
	{
		JButton setButton = new JButton( "Set" );
		//setButton.setPreferredSize( new Dimension( 40, 24 ) );

		return setButton;
	}

	private BlockInputStream blockin;

	/**
	 *
	 * */
	public Main( Map props )
	{
		stayOpenDefault = ((Boolean)props.get( KEY_persist )).booleanValue();
		
		blockin = new BlockInputStream( System.in, OutThrottle.MAX_bufsize );
		throttle = new OutThrottle( blockin, System.out );

		throttle.setDesiredRate( (Rate)props.get(KEY_desiredRate) );
	
		throttle.addObserver( new Observer()
				{
					public void update( Observable o1, Object o2 )
					{
						// Make a code to do the update, that runs in
						// the swing thread
						Runnable updateCode = new Runnable()
						{
							public void run()
							{
								updateLabels();
							}
						};

						try
						{
							SwingUtilities.invokeLater( updateCode );
						}
						catch( Exception ie )
						{
							return;
						}

					}
				});


		// Initialize important components
		initComponents();

		// Lay out the components
		mainComponent = makeLayout(false);

		add( mainComponent );
	}


	private JComponent makeLayout(boolean longform)
	{
		JPanel bigPanel = new JPanel();
		
		// Make a list (long form)
		activeComponents = makeComponents(longform);

		// +1 for the button row
		int rows=activeComponents.size()+2;

		// 2 columns, each 50% size
		double columnSizes[] = {0.50, 0.50};

		double rowSizes[] = new double[rows];

		for( int i = 0; i < rows; ++i )
		{
			// Size every row to whatever it wants
			rowSizes[i]=TableLayout.PREFERRED;
		}

		double size[][] =
			{ columnSizes, rowSizes };

		TableLayout layout = new TableLayout( size );
		bigPanel.setLayout( layout );


		rows=0;
		JLabel label;

		JComponent jc;

		
		Iterator compI = activeComponents.iterator();
		while( compI.hasNext() )
		{
			Component comp = (Component)compI.next();
			bigPanel.add( comp, "0, "+rows+", 1, "+rows );
			++rows;
		}


		pauseBtn = new JButton("Pause");
		pauseBtn.addActionListener( new ActionListener()
				{
					public void actionPerformed( ActionEvent ae )
					{
						throttle.togglePaused();
					}
				});

		stopBtn = new JButton( "Stop" );
		stopBtn.addActionListener( new ActionListener()
				{
					public void actionPerformed( ActionEvent ae )
					{
						System.exit(0);
					}
				});
		jc = makeTwo( pauseBtn, stopBtn );
		bigPanel.add( jc, "0, "+rows+", 1, "+rows );
		++rows;


		return bigPanel;
	}


	private JButton showLessButton;
	private JButton showMoreButton;


	private JButton makeMoreButton()
	{
		if( showMoreButton != null )
			return showMoreButton;

		showMoreButton = makeChangeSizeButton(true);
		return showMoreButton;
	}

	private JButton makeLessButton()
	{
		if( showLessButton != null )
			return showLessButton;

		showLessButton = makeChangeSizeButton(false);
		return showLessButton;
	}

	private JButton makeChangeSizeButton(final boolean longform)
	{
		JButton jb = new JButton(longform?"Show more":"Show less");
		jb.addActionListener(new ActionListener()
				{
					public void actionPerformed( ActionEvent ae )
					{
						// 
						Component newC = makeLayout(longform);
						remove( mainComponent );
						mainComponent = newC;
						add( mainComponent );
					}
				});

		return jb;
	}

	/**
	 *
	 * */
	private JComponent makeTwo( String label, JComponent comb )
	{
		return makeTwo( new JLabel( label ), comb );
	}
	private JComponent makeTwo( final JComponent coma, final JComponent comb )
	{
		double columnSizes[] = {0.50, 0.50};
		double size2[][] =
			{ columnSizes, new double[]{TableLayout.PREFERRED} };
		TableLayout row = new TableLayout(size2);

		JPanel jp = new JPanel()
		{
			public void setEnabled( boolean bool )
			{
				super.setEnabled( bool );
				coma.setEnabled( bool );
				comb.setEnabled( bool );
			}
		};
		jp.setLayout( row );

		jp.add( coma, "0,0" );
		jp.add( comb, "1,0" );

		return jp;
	}

	private void initComponents()
	{

		// Bytes
		bytesLabel = new JLabel("-");

		// Rate label
		rateLabel = new JLabel("-");

		// Rate selection
		desiredRateSelect = new JRateSelect(throttle.getDesiredRate());
		desiredRateSelect.addActionListener( this );

		// The Window
		windowLabel = new JProgressBar();

		// The desired window
		desiredWindowSelect = new JWindowSelect((int)throttle.getDesiredWindow());
		desiredWindowSelect.addActionListener( this );

		// Buffer size (kind of a joke actually)
		bufLabel = new JLabel("-");

		// Amount of data in buffer
		bufFillLabel = new JLabel("-");

		// The biggest write performed in the window
		windowMaxReadLabel = new JLabel("-");

		// The time taken
		timeLabel = new JLabel("-");

		// Do we stay open?
		stayOpenBox = new JCheckBox( "Stay open after pipe closes", stayOpenDefault );
		stayOpenBox.addActionListener( this );
	}

	/**
	 * @return a list of components to use
	 * */
	private List makeComponents(boolean longform)
	{
		List l = new LinkedList();
		Component jc;

		jc = makeTwo("Bytes: ", bytesLabel );
		l.add(jc);

		jc = makeTwo("Rate: ", rateLabel );
		l.add(jc);

		jc = makeTwo("Desired rate: ", desiredRateSelect );
		l.add(jc);

		//l.add(Box.createGlue());

		if( longform )
		{
			jc = makeTwo("Window:", windowLabel);
			l.add(jc);

			jc = makeTwo("Desired Window:", desiredWindowSelect);
			l.add(jc);

			jc = makeTwo("Buffer:", bufLabel);
			l.add(jc);

			jc = makeTwo("Buffer fill:", bufFillLabel);
			l.add(jc);

			jc = makeTwo( "Max write:", windowMaxReadLabel );
			l.add(jc);
		}

		jc = makeTwo( "Time: ", timeLabel );
		l.add(jc);

		JPanel jp = new JPanel();
		if( longform )
			jp.add( makeLessButton() );
		else
			jp.add( makeMoreButton() );
		
		JPanel p = new JPanel();
		p.setLayout( new TableLayout(new double[][]{{TableLayout.FILL},{TableLayout.FILL}}) );
		p.add( stayOpenBox, "0,0,c,c");

		jc = makeTwo( p, jp );
		l.add( jc );
		
		jc = Box.createVerticalStrut(10);
		l.add(jc);

		return l;
	}

	public void actionPerformed( ActionEvent ae )
	{
		if( ae instanceof NewRateEvent )
		{
			NewRateEvent nre = (NewRateEvent)ae;

			throttle.setDesiredRate(nre.getNewRate());
		}
		else if( ae instanceof NewWindowEvent )
		{
			NewWindowEvent nwe = (NewWindowEvent)ae;
			int newW = nwe.getNewWindow(); 


			// Window has to be able to record at least one byte
			if( newW < throttle.getDesiredRate().getMSecPerByte() )
			{
				newW = (int)throttle.getDesiredRate().getMSecPerByte();
				desiredWindowSelect.setWindow( newW/1000 );
			}

			throttle.setDesiredWindow(newW);
			windowLabel.setMaximum( newW );
		}
		//else if( ae.getSource() == stayOpenBox )
		//{
		//	stayOpenOnClose = 
		//}
	}

	public void run()
	{
		throttle.start();
	}

	public static void usage()
	{
		log( "Usage: " );
		log( "   java -jar javathrottle.jar <options>" );
		log( "" );
		log( " Options:" );
		log( "  -b, -k, -m <number>" );
		log( "      Limit pipe in bits, kilobits, or megabits per second" );
		log( "  -B, -K, -M <number>" );
		log( "      Limit pipe in bytes, kilobytes, or megabytes per second" );
		log( "  -U" );
		log( "      Start with throttle wide open" );
		log( "  --persist" );
		log( "      Keep window open after pipe closes" );
		log( "" );
		log( "  --debug " );
		log( "      Enable debugging output");
		log( "  --version " );
		log( "      Show version number ");

	}

	public static void debug(String msg)
	{
		if( DEBUG_enable )
			System.err.println(msg);
	}

	public void stop()
	{
		//throttle.shutdown();
		System.exit(0);
	}

	// ------------ private methods --------------
	private static Properties parseCmdLine( String args[] )
	{
		Properties p = new Properties();

		Unit unit = null;
		double value = 1;
		boolean persist = false;

		for( int i = 0; i < args.length; ++i )
		{
			if( args[i].equals( "--version" ) )
			{
				log( VER_project + " " + VER_version );
				log( VER_author );
				log( VER_url );
				System.exit(0);
			}
			else if( args[i].equals( "--debug" ) )
			{
				DEBUG_enable = true;
				continue;
			}
			else if( args[i].equals( "--persist" ) )
			{
				persist = true;
				continue;
			}

			// Any argument must be in the form "-X"
			if( args[i].length() < 2 ) 
				return null;

			// If there are any flags, they must start with "-"
			if( args[i].charAt(0) != '-' )
				return null;

			char c = args[i].charAt(1);
			boolean readRate = false;
			if( readRate = c == 'b' )
				unit = Unit.bps;
			else if( readRate = c == 'k' )
				unit = Unit.kbps;
			else if ( readRate = c == 'm' )
				unit = Unit.Mbps;
			else if( readRate = c == 'B' )
				unit = Unit.Bps;
			else if( readRate = c == 'K' )
				unit = Unit.kBps;
			else if( readRate = c == 'M' )
				unit = Unit.MBps;
			else if( c == 'U' )
			{
				unit = Unit.unlimited;
				p.put( KEY_desiredRate, unit.makeRate( 1 ) );
			}
			else return null;

			if( readRate )
			{
				p.put( KEY_desiredRate, unit.makeRate( args[++i] ) );
			}
		}

		p.put( KEY_persist, new Boolean( persist ) );

		return p;
	}

	private static void log(String msg)
	{
		System.err.println(msg);
	}

	private void updateLabels()
	{
		bufLabel.setText( "" + throttle.getActualBufSize() );
		rateLabel.setText( "" + throttle.getActualRate().toString() );
		bytesLabel.setText( "" + throttle.getTotalBytes() );
		windowLabel.setValue( (int)throttle.getActualWindow() );
		bufFillLabel.setText( "" + blockin.getFill() );
		windowMaxReadLabel.setText( "" + throttle.getMaxWindowRead() );

		// Time label
		long now = System.currentTimeMillis();
		long totalTime = throttle.getTotalTime();
		if( totalTime > 0 )
			timeLabel.setText( elapsedTimeToStr( totalTime ) );

		// Check if the stream is over
		if( throttle.isDone() )
		{
			if( !stayOpenBox.isSelected() )
				System.exit(0);

			stopBtn.setText("Quit");
			pauseBtn.setEnabled(false);

			Iterator it = activeComponents.iterator();
			while( it.hasNext() )
			{
				Component comp = (Component)it.next();
				comp.setEnabled( false );
			}

			//bufLabel.setEnabled(false);
			//rateLabel.setEnabled(false);
			//timeLabel.setEnabled(false);
			//bytesLabel.setEnabled(false);
			//windowLabel.setEnabled(false);
			//stayOpenBox.setEnabled(false);
			//desiredRateSelect.setEnabled(false);
			//windowMaxReadLabel.setEnabled(false);
			//desiredWindowSelect.setEnabled(false);
			windowLabel.setValue(Integer.MAX_VALUE);
		}
	}


	private String elapsedTimeToStr( long elapsed )
	{
		long seconds = elapsed / 1000;
		long minutes = seconds / 60;
		long hours = minutes / 60;
		long days = hours / 24;

		seconds = seconds % 60;
		minutes = minutes % 60;
		hours = hours % 24;

		StringBuffer sb = new StringBuffer(20);
		boolean div = false;
		div = appendTimeUnit( sb, days, " day", " days", div );
		div = appendTimeUnit( sb, hours, " hour", " hours", div );
		div = appendTimeUnit( sb, minutes, " minute", " minutes", div );
		div = appendTimeUnit( sb, seconds, " second", " seconds", div );

		return sb.toString();
	}

	private boolean appendTimeUnit( StringBuffer sb, long data, String unary, String plural, boolean prefixDelim )
	{
		if( data > 0 )
		{
			if( prefixDelim )
				sb.append( ", " );

			sb.append( "" + data );
			if( data > 1 )
				sb.append( plural );
			else
				sb.append( unary );
			return true;
		}
		else
			return prefixDelim;
	}
}

==> javathrottle-0.5.tar.gz: javathrottle-0.5/src/net/f00f/javathrottle/Main.java <==
package net.f00f.javathrottle;
/*
	JavaThrottle, a program to limit the bandwidth between an input
	stream and an output stream.  Copyright (C) 2003 Ben Damm

	For information, contact bdamm@f00f.net

                        -----  LGPL License  -----
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                        -----       END      -----

*/


import java.awt.event.ContainerAdapter;
import java.awt.event.ContainerEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import net.f00f.javathrottle.gui.GuiPanel;

/**
 * This class does the command line switches and starts the GUI.  The
 * throttling of the pipe is done in a Throttle implementation.
 *
 * @author Ben Damm
 * */

public class Main
{
	public static String KEY_desiredRate = "desiredRate";
	public static String KEY_persist = "persist";

	private Throttle throttle;

	private static String VER_project = "JavaThrottle";
	private static String VER_version = Version.VERSION + " " + Version.BUILD_DATE;
	private static String VER_author = "Ben Damm, bdamm@f00f.net";
	private static String VER_url = "http://dammfine.com/projects/javathrottle/";

	private static boolean DEBUG_enable = true;

	/**
	 * Entry point.
	 * */
	public static void main( String args[] )
	{
		// Parse cmd line
		final Properties p = parseCmdLine( args );

		// Error
		if( p == null )
		{
			usage();
			return;
		}

		// The window skeleton
		final JFrame jf = new JFrame("JavaThrottle");

		// When the user closes the window, kill the application.
		jf.addWindowListener(new WindowAdapter()
				{
					public void windowClosing( WindowEvent we )
					{
						System.exit(0);
					}
				});

		Runnable runner = new Runnable()
		{
			public void run()
			{
				// Create the main class
				GuiPanel mainPane = new GuiPanel(p);

				mainPane.addContainerListener(new ContainerAdapter()
						{
							public void componentAdded( ContainerEvent ce )
							{
								// Resize to fit the components
								jf.pack();
							}
						});

				// Add it
				jf.getContentPane().add( mainPane );
				jf.pack();
				jf.show();

				// Start the magic
				mainPane.run();
			}
		};

		try
		{
			SwingUtilities.invokeAndWait( runner );
		}
		catch( java.lang.reflect.InvocationTargetException ite )
		{
			// ??
		}
		catch( InterruptedException ie )
		{
			return;
		}
	}

	public static void usage()
	{
		log( "Usage: " );
		log( "   java -jar javathrottle.jar <options>" );
		log( "" );
		log( " Options:" );
		log( "  -b, -k, -m <number>" );
		log( "      Limit pipe in bits, kilobits, or megabits per second" );
		log( "  -B, -K, -M <number>" );
		log( "      Limit pipe in bytes, kilobytes, or megabytes per second" );
		log( "  -U" );
		log( "      Start with throttle wide open" );
		log( "  --persist" );
		log( "      Keep window open after pipe closes" );
		log( "" );
		log( "  --debug " );
		log( "      Enable debugging output");
		log( "  --version " );
		log( "      Show version number ");

	}

	public static void debug(String msg)
	{
		if( DEBUG_enable )
			System.err.println(msg);
	}

	public void stop()
	{
		System.exit(0);
	}

	// ------------ private methods --------------
	private static Properties parseCmdLine( String args[] )
	{
		Properties p = new Properties();

		Unit unit = null;
		double value = 1;
		boolean persist = false;

		for( int i = 0; i < args.length; ++i )
		{
			if( args[i].equals( "--version" ) )
			{
				log( VER_project + " " + VER_version );
				log( VER_author );
				log( VER_url );
				System.exit(0);
			}
			else if( args[i].equals( "--debug" ) )
			{
				DEBUG_enable = true;
				continue;
			}
			else if( args[i].equals( "--persist" ) )
			{
				persist = true;
				continue;
			}

			// Any argument must be in the form "-X"
			if( args[i].length() < 2 ) 
				return null;

			// If there are any flags, they must start with "-"
			if( args[i].charAt(0) != '-' )
				return null;

			char c = args[i].charAt(1);
			boolean readRate = false;
			if( readRate = c == 'b' )
				unit = Unit.bps;
			else if( readRate = c == 'k' )
				unit = Unit.kbps;
			else if ( readRate = c == 'm' )
				unit = Unit.Mbps;
			else if( readRate = c == 'B' )
				unit = Unit.Bps;
			else if( readRate = c == 'K' )
				unit = Unit.kBps;
			else if( readRate = c == 'M' )
				unit = Unit.MBps;
			else if( c == 'U' )
			{
				unit = Unit.unlimited;
				p.put( KEY_desiredRate, unit.makeRate( 1 ) );
			}
			else return null;

			if( readRate )
			{
				p.put( KEY_desiredRate, unit.makeRate( args[++i] ) );
			}
		}

		p.put( KEY_persist, new Boolean( persist ) );

		return p;
	}

	private static void log(String msg)
	{
		logError( msg );
	}

	public static void logError(String msg)
	{
		System.err.println(msg);
	}
}
